// file: BinarySearch.java
// author: Sedgewick and Wayne
// modified by: Bob Muller
//   The alterations presume a sorted whitelist as input and merely
//   counts the misses while running a Stopwatch.
//
// date: February 13, 2014
//
// 
/*************************************************************************
 *  Compilation:  javac BinarySearch.java
 *  Execution:    java BinarySearch whitelist.txt < input.txt
 *  Data files:   http://algs4.cs.princeton.edu/11model/tinyW.txt
 *                http://algs4.cs.princeton.edu/11model/tinyT.txt
 *                http://algs4.cs.princeton.edu/11model/largeW.txt
 *                http://algs4.cs.princeton.edu/11model/largeT.txt
 *
 *  % java BinarySearch tinyW.txt < tinyT.txt
 *  50
 *  99
 *  13
 *
 *  % java BinarySearch largeW.txt < largeT.txt | more
 *  499569
 *  984875
 *  295754
 *  207807
 *  140925
 *  161828
 *  [3,675,966 total values]
 *  
 *************************************************************************/

import java.util.Arrays;

/**
 *  The <tt>BinarySearch</tt> class provides a static method for binary
 *  searching for an integer in a sorted array of integers.
 *  <p>
 *  The <em>rank</em> operations takes logarithmic time in the worst case.
 *  <p>
 *  For additional documentation, see <a href="http://algs4.cs.princeton.edu/11model">Section 1.1</a> of
 *  <i>Algorithms, 4th Edition</i> by Robert Sedgewick and Kevin Wayne.
 *
 *  @author Robert Sedgewick
 *  @author Kevin Wayne
 */
public class BinarySearch {

    /**
     * This class should not be instantiated.
     */
    private BinarySearch() { }

    /**
     * Searches for the integer key in the sorted array a[].
     * @param key the search key
     * @param a the array of integers, must be sorted in ascending order
     * @return index of key in array a[] if present; -1 if not present
     */
    public static int rank(int key, int[] a, int lo, int hi) {

        if (lo <= hi) {
            int mid = lo + (hi - lo) / 2;
            int result = a[mid] - key;
            if (result > 0)
                return rank(key, a, lo, mid - 1);
            else if (result < 0)
                return rank(key, a, mid + 1, hi);
            else 
                return mid;  // found key in a[mid].
        }
        return -1;
    }

    /**
     * Reads in a sequence of integers from the whitelist file, specified as
     * a command-line argument. Reads in integers from standard input and
     * prints to standard output those integers that do *not* appear in the file.
     */
    public static void main(String[] args) {

        // read the integers from a file
        In in = new In(args[0]);
        int[] whitelist = in.readAllInts();

        int count = 0;

        // Muller: we assume the input is already sorted!
        // Arrays.sort(whitelist);

        // read integer key from standard input; print if not in whitelist
        // Muller:
        Stopwatch sw = new Stopwatch();

        while (!StdIn.isEmpty()) {
            int key = StdIn.readInt();
            if (rank(key, whitelist, 0, whitelist.length - 1) == -1)
                // Muller
                //
                count++;
        }
        // Muller
        //
        System.out.println("count= " + count + ", time= " + sw.elapsedTime());
    }
}